"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var nodes_1 = require("rev-models/lib/queries/nodes");
var utils_1 = require("rev-models/lib/queries/utils");
var queryparser_1 = require("rev-models/lib/queries/queryparser");
var FIELD_OPERATOR_MAPPING = {
    eq: '$eq',
    ne: '$ne',
    gt: '$gt',
    gte: '$gte',
    lt: '$lt',
    lte: '$lte',
    like: '__like',
    in: '$in',
    nin: '$nin'
};
function _convertConjunctionNode(srcNode, destNode) {
    if (srcNode.children.length == 0) {
        return;
    }
    var mongoOperator = (srcNode.operator == 'and' && '$and') || (srcNode.operator == 'or' && '$or');
    if (!mongoOperator) {
        throw new Error("Conjunction Operator '" + srcNode.operator + "' not recognised.");
    }
    destNode[mongoOperator] = srcNode.children.map(function (child) {
        var childDestNode = {};
        _convertQueryNode(child, childDestNode);
        return childDestNode;
    });
}
function _convertFieldNode(srcNode, destNode) {
    destNode[srcNode.fieldName] = {};
    for (var _i = 0, _a = srcNode.children; _i < _a.length; _i++) {
        var valueNode = _a[_i];
        var mongoOperator = FIELD_OPERATOR_MAPPING[valueNode.operator];
        if (!mongoOperator) {
            throw new Error("Field Value Operator '" + valueNode.operator + "' not recognised.");
        }
        if (valueNode instanceof nodes_1.ValueOperator) {
            var value = valueNode.value;
            if (mongoOperator == '__like') {
                mongoOperator = '$regex';
                value = utils_1.getLikeOperatorRegExp(value);
            }
            destNode[srcNode.fieldName][mongoOperator] = value;
        }
        else if (valueNode instanceof nodes_1.ValueListOperator) {
            destNode[srcNode.fieldName][mongoOperator] = valueNode.values;
        }
        else {
            throw new Error("Field Value Operator '" + valueNode.operator + "' not supported.");
        }
    }
}
function _convertQueryNode(srcNode, destNode) {
    if (srcNode instanceof nodes_1.ConjunctionNode) {
        _convertConjunctionNode(srcNode, destNode);
    }
    else if (srcNode instanceof nodes_1.FieldNode) {
        return _convertFieldNode(srcNode, destNode);
    }
    else {
        throw new Error("Unrecognised Query Node: " + srcNode.constructor.name);
    }
}
/**
 * @private
 */
function convertQuery(manager, model, where) {
    var parser = new queryparser_1.QueryParser(manager);
    var queryNode = parser.getQueryNodeForQuery(model, where);
    var mongoQuery = {};
    _convertQueryNode(queryNode, mongoQuery);
    return mongoQuery;
}
exports.convertQuery = convertQuery;
//# sourceMappingURL=query.js.map